Tokenization with Split Trees

Fuente: arXiv
Enregistré dans:
Détails bibliographiques
Auteurs principaux: Schmidt, Craig W., Krumdick, Michael, Wiemerslage, Adam, Ebner, Seth, Reddy, Varshini, Pinter, Yuval, Tanner, Chris
Format: Preprint
Publié: 2026
Sujets:
Accès en ligne:
Tags: Ajouter un tag
Pas de tags, Soyez le premier à ajouter un tag!
_version_ 1866914606409056256
author Schmidt, Craig W.
Krumdick, Michael
Wiemerslage, Adam
Ebner, Seth
Reddy, Varshini
Pinter, Yuval
Tanner, Chris
author_facet Schmidt, Craig W.
Krumdick, Michael
Wiemerslage, Adam
Ebner, Seth
Reddy, Varshini
Pinter, Yuval
Tanner, Chris
contents We introduce Tokenization with Split Trees (ToaST), a subword tokenization method that directly optimizes compression under a new recursive inference procedure. ToaST greedily splits each pretoken into a full binary tree using precomputed byte n-gram counts, independent of any vocabulary. Given a vocabulary, inference recursively descends each split tree and emits the first in-vocabulary node reached on each path. Vocabulary selection is formulated as an Integer Program (IP) that minimizes the total token count over all split trees under this inference procedure. The Linear Programming (LP) relaxation is near-integral in practice, yielding provably near-optimal vocabularies, with training time empirically scaling quadratically in the number of split trees. On English text, ToaST reduces token counts by more than 11% compared to BPE, WordPiece, and UnigramLM at vocabulary sizes of 40,960 and above, reducing the number of inference tokens for models using this tokenizer, thus extending the effective context length. ToaST also uses common single-byte tokens less frequently than these baselines, leading to a substantial improvement in Renyi efficiency. In experiments training 1.5B parameter language models, ToaST achieves the highest CORE score, outperforming baselines by 2.6%--7.6%, with significance for two of three, and scoring best on 13 of 22 individual tasks.
format Preprint
id arxiv_https___arxiv_org_abs_2605_22705
institution arXiv
publishDate 2026
record_format arxiv
spellingShingle Tokenization with Split Trees
Schmidt, Craig W.
Krumdick, Michael
Wiemerslage, Adam
Ebner, Seth
Reddy, Varshini
Pinter, Yuval
Tanner, Chris
Computation and Language
We introduce Tokenization with Split Trees (ToaST), a subword tokenization method that directly optimizes compression under a new recursive inference procedure. ToaST greedily splits each pretoken into a full binary tree using precomputed byte n-gram counts, independent of any vocabulary. Given a vocabulary, inference recursively descends each split tree and emits the first in-vocabulary node reached on each path. Vocabulary selection is formulated as an Integer Program (IP) that minimizes the total token count over all split trees under this inference procedure. The Linear Programming (LP) relaxation is near-integral in practice, yielding provably near-optimal vocabularies, with training time empirically scaling quadratically in the number of split trees. On English text, ToaST reduces token counts by more than 11% compared to BPE, WordPiece, and UnigramLM at vocabulary sizes of 40,960 and above, reducing the number of inference tokens for models using this tokenizer, thus extending the effective context length. ToaST also uses common single-byte tokens less frequently than these baselines, leading to a substantial improvement in Renyi efficiency. In experiments training 1.5B parameter language models, ToaST achieves the highest CORE score, outperforming baselines by 2.6%--7.6%, with significance for two of three, and scoring best on 13 of 22 individual tasks.
title Tokenization with Split Trees
topic Computation and Language
url https://arxiv.org/abs/2605.22705